
// Robot puppet controller by John Blankenship & Samuel Mishal
// NOTE: This program utilizes the routines in our Puppet Control Program
//       which controlled the puppet using an extended joystick.
//       If the routines seem strange at times, it is because they were
//       originally developed for the joystick.  Our goal here was to show
//       some basic vision capabilities, so very little was done to the 
//       original hardware control routines for the puppet.
// Parallax USB servo controller used to control 7 servos in puppet.
// Once device is installed, note the assigned port and alter
//      the following line if necessary.

   COMMS_PORT     = 6 
   // serial port number assigned to the USB servo controller
//====================================================================
//====================================================================
MainProgram:
  GoSub Instructions
  Gosub SetColors
  Gosub SetUp
  GoSub Initialize
  GoSub Start_Control
End
//====================================================================
SetUp:
  n=ErrMsg(Msgs[1],Msgs[0],MB_QUESTION|MB_YesNo|MB_Button2)
  if n=MB_Yes then GoSub Initialize_Comms
  GoSub Establish_Comms
Return
//====================================================================
Initialize:
   // set these items to establish parameter for each action
   // Channel refers to the Parallax servo controller
   // LowLimit and HighLimit sets extent of servo momvement
   // Normal is the "centered" position
   // Randomness is the maximum amount of random servo movement 
   // Ramping effectively sets the startup speed 0-fast 10 or more-very slow

   rArm = 0 \rTorso = 0 \rHeadR = 0 \ rHeadT = 0 // use for random movements
   //---Hat L/R buttons

   EyesChannel=14 \ EyesLowLimit=500 \ EyesHighLimit=800 
   EyesRamping=7 \ EyesNormal=650   \ EyesRandomness=0

   //--Button 1
   B_Mouth = 1
   MouthChannel=2 \ MouthLowLimit=650 \ MouthHighLimit=910 
   MouthRamping=1 \ MouthNormal=700   \ MouthRandomness=0

   //--Hat U Buttons
   H_Brows = 0
   BrowsChannel=4 \ BrowsLowLimit=600 \ BrowsHighLimit=800 
   BrowsRamping=7 \ BrowsNormal=700   \ BrowsRandomness=0

   //---R axis
   HeadRChannel=6 \ HeadRLowLimit=900 \ HeadRHighLimit=500 
   HeadRRamping=9 \ HeadRNormal=700   \ HeadRRandomness=30
   HeadRTime = timer()
   
   //---Y axis
   HeadTChannel=8 \ HeadTLowLimit=1000 \ HeadTHighLimit = 700 
   HeadTRamping=9 \ HeadTNormal=950   \ HeadTRandomness=50
   HeadTTime = timer()

   //---X axis
   TorsoChannel=10 \ TorsoLowLimit=620 \ TorsoHighLimit=800 
   TorsoRamping=15  \ TorsoNormal=720   \ TorsoRandomness=50
   TorsoTime = timer()

   //--Button 2
   B_Arms = 2
   ArmsChannel=12 \ ArmsLowLimit=300 \ ArmsHighLimit=700 
   ArmsRamping=10  \ ArmsNormal=700   \ ArmsRandomness=100
   ArmsTime = timer()
   
   //---Combination Movements Buttons
   B_Combo1 = 4 \ B_Combo2 = 8 \ B_Yes = 16 \ B_No = 32
   
   rArms = 0
   // create an array of search colors
  // dim light Data SearchClrs; 3771547, 4415244, 6711774
 //  data SearchClrs; 4777157, 7591691, 5337342
   //               yellow    green    red 
   
   
   // create an array of sound files to say each item
   Data SpeakColors; "C:\Users\John\Documents\Yellow"
   Data SpeakColors; "C:\Users\John\Documents\Green"
   Data SpeakColors; "C:\Users\John\Documents\Red"
   
   LeftRt = .5  // head
   Tilt = .5    // head
   Body = .5    // body tilt
   Arms = 0
   Normal = -1
   Mouth = Normal
   Eyes = Normal    
   Brows = Normal
     LookLeft = 270
     LookRight = 90
     MoveUp = 180
     MoveDown = 0
     Open = 1
   NotSaidBefore = True
   oldcc = -1
   delay 1000
   // initialize vision variables
   Dim files[50]
   Dir = "C:\Users\John\Documents\Live! Cam Center\Remote Monitoring\20080728\"
Return
//====================================================================
Instructions:
  data Msgs;"Puppet.Bas","Do you want to intialize the controller board?"
  data I_M;Msgs[0],"The puppet control program"
  //n=MsgBox(I_M)
Return
//====================================================================
Initialize_Comms:
   // This section MUST be customized for your servo controller
   // first set up communication to match the default for the 
   //    servo controller
   // note: This should only be done the first time the program is run
   //  (which is why you are given the option of Initializing each time)
   //  but... it works fine on my machine to Initialize every time
   //  probably because the low baud transmission is just ignored
   //  it has been left in case there are problem on YOUR machine
   //---establish low speed comms
   setCommPort COMMS_PORT, br2400
   SerOut "!SCVER?",char(13)
   delay 200
   SerIn S
   // now set the Servo Controller's communication to maximum baud rate 
   //----set baud rate to 38400 baud
   SerOut "!SCSBR",char(1),char(13)
   delay 200
return
//====================================================================
Establish_Comms:
   //---establish high speed comms (set the PC to the higher baud rate)
   setCommPort COMMS_PORT, br38400
   SerOut "!SCVER?",char(13)
   delay 200
   SerIn S
Return
//====================================================================

//====================================================================
Start_Control:
   while true 
      x=CaptureImage()
     // reSizeBmp "",50,50
      FitCB   
      // see if ANY of the three expected objects (colors) is within view
      for cc=0 to 2
        searchClr = SearchClrs[cc]
        BmpFindClr "",searchClr,Found,SecCnt,.1,.1,5,CntInfo
        if Found>-1 then break  // keep the first one found
      next
      // if nothing seen show discust
      if Found = -1
          //the colors is lost
          Brows = MoveDown 
          GoSub Brows_Movements
      endif
      // if found, track that one color
      while Found>-1
          // color has been found... now adjust the puppet's variables to
          // look toward the object (based on its position on the screen)
          x = Found#5
          y = Found/5
          //print "x,y = ",x;y
          linewidth 10
          erectangle x*160,y*120,x*160+160,y*120+120,10,searchClr
          Eyes=Normal
          Brows = Normal
          if x<1 
             LeftRt = LeftRt-.1
             if LeftRt<.1 then LeftRt=.1
             Eyes = LookLeft
             Brows = MoveUp
          endif
          if x<2 
             LeftRt = LeftRt-.05
             if LeftRt<.1 then LeftRt=.1
             Eyes = LookLeft
             Brows = MoveUp
          endif
          if x>2
             LeftRt = LeftRt+.05
             if LeftRt>1 then LeftRt = 1
             Eyes=LookRight
             Brows = MoveUp
          endif
          if x>3
             LeftRt = LeftRt+.1
             if LeftRt>1 then LeftRt = 1
             Eyes=LookRight
             Brows = MoveUp
          endif
          if y>2 
            Tilt = Tilt-.1
            if Tilt<.1 then Tilt = .1
            Brows = MoveUp
          endif
          if y>3 
            Tilt = Tilt-.1
            if Tilt<.1 then Tilt = .1
            Brows = MoveUp
          endif
          if y<1 
            Tilt = Tilt+.1
            if Tilt>1 then Tilt=1
            Brows = MoveUp
          endif
          if y<2 
            Tilt = Tilt+.1
            if Tilt>1 then Tilt=1
            Brows = MoveUp
          endif
   
          // now actually move the puppet       
          GoSub HeadR_Movements
          GoSub HeadT_Movements
          GoSub Arms_Movements
          GoSub Brows_Movements
          GoSub Eyes_Movements
          GoSub Mouth_Movements
          GoSub Torso_Movements
        
          // now say object if centered  
          if x=2 and y=2
            PlayWav SpeakColors[cc]
            delay 400
            for ii = 1 to 7
              Mouth = Open
              GoSub Mouth_Movements
              delay 150
              Mouth = Normal
              GoSub Mouth_Movements
              delay 150
            next
          endif    
          x=CaptureImage()
         // reSizeBmp "",50,50
          FitCB
          // see if the EXPECTED object is within view
          BmpFindClr "",searchClr,Found,SecCnt,.2,.1,5,CntInfo 
      wend 
   wend
Return
//====================================================================

//====================================================================
HeadR_Movements:
   delay 10
   // rotate the head
   // Control variable = LeftRt
   ramping = HeadRRamping   
   channel = HeadRChannel
   if HeadRTime < Timer()
     rHeadR =random(HeadRRandomness)-HeadRRandomness/2
     HeadRTime = Timer()+1000 // random movement every 1000 mseconds
   endif
   nn = HeadRLowLimit+round(LeftRt*(HeadRHighLimit-HeadRLowLimit))+rHeadR
   SerOut "!SC",char(channel),char(ramping),char(nn&255),char((nn >> 8)&255),char(13)
Return
//====================================================================
HeadT_Movements:
   delay 10
   // tilt the head
   // control variable = Tilt
   ramping = HeadTRamping   
   channel = HeadTChannel
   if HeadTTime < Timer()
     rHeadT =random(HeadTRandomness)-HeadTRandomness/2
     HeadTTime = Timer()+1000 // random movement every 1000 mseconds
   endif
   nn = HeadTLowLimit+round(Tilt*(HeadTHighLimit-HeadTLowLimit))+rHeadT
   SerOut "!SC",char(channel),char(ramping),char(nn&255),char((nn >> 8)&255),char(13)
Return
//====================================================================
Arms_Movements:
   delay 10
   // control variable = Arms
   ramping = ArmsRamping
   channel = ArmsChannel
   if ArmsTime < Timer()
     rArms =random(ArmsRandomness)-ArmsRandomness/2
     ArmsTime = Timer()+3000 // random movment every 3000 mseconds
   endif
   if Arms == 1 
     n = ArmsHighLimit
   else 
     n = ArmsLowLimit + rArms
   endif   
   SerOut "!SC",char(channel),char(ramping),char(n&255),char((n >> 8)&255),char(13)
Return
//====================================================================
Eyes_Movements:
   delay 10
   // control variable = Eyes
   ramping = EyesRamping
   channel = EyesChannel
   n = EyesNormal
   if Eyes == 270 then n = EyesHighLimit
   if Eyes == 90 then n = EyesLowLimit
   SerOut "!SC",char(channel),char(ramping),char(n&255),char((n >> 8)&255),char(13)
Return
//====================================================================
Brows_Movements:
   delay 10
   // Control variable = Brows
   ramping = BrowsRamping
   channel = BrowsChannel
   n = BrowsNormal
   if Brows == 0 then n = BrowsHighLimit
   if Brows == 180 then n = BrowsLowLimit
   SerOut "!SC",char(channel),char(ramping),char(n&255),char((n >> 8)&255),char(13)
Return
//====================================================================
Mouth_Movements:
   delay 10
   // control variable = Mouth
   ramping = MouthRamping
   channel = MouthChannel
   n = MouthLowLimit
   if Mouth = 1 then n = MouthHighLimit
   SerOut "!SC",char(channel),char(ramping),char(n&255),char((n >> 8)&255),char(13)
Return
//====================================================================
Torso_Movements:
   delay 10
   // control variable = Body
   ramping = TorsoRamping   
   channel = TorsoChannel
   if TorsoTime < Timer()
     rTorso =random(TorsoRandomness)-TorsoRandomness/2
     TorsoTime = Timer()+1000 // random movement every 1000 mseconds
   endif
   nn = TorsoLowLimit+round(Body*(TorsoHighLimit-TorsoLowLimit))+rTorso
   SerOut "!SC",char(channel),char(ramping),char(nn&255),char((nn >> 8)&255),char(13)
Return
//====================================================================
SetColors:
  Print "First, choose camera and set its resolution"
  x=CaptureSrc()
  x=CaptureDlg()
  Dim SearchClrs[3]
 // clearscr
  Print "The robot in this demonstration will recognize THREE objects by color"
  Print
  Print "In my case they were Fish Food, a Christmas Tree, and a Veggie Brush)"
  Print "Get three objects (Yellow, Green, and Red)"
  Print "Ideally, these objects will have unique colors compared to the room objets"
  Print
  Print "Be prepared to hold them all in front of the robot's camera"
  Print "After taking a picture of the objects, you will be asked to click each one"
  Print "    with the mouse (to identify the colors to be searched for)."
  Print
  Print "Press any key to start the count down to take a picture"
  Input "Press any key",i
  For i = 3 to 0
    delay 1000
    clearscr
   XYText 400,300,i,"Times New Roman",50,fs_Bold
  next
  x = CaptureImage()
  //reSizeBmp "",50,50
  FitCB
  print "Click YELLOW, then GREEN, then RED"
  for i = 0 to 2
    repeat
      ReadMouse x,y,b
    until b =1
    delay 1000
    ReadPixel x,y,cc
    SearchClrs[i]=cc
    print SearchClrs[i]
  next
return

//====================================================================

